// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================


/**
 * @file MSSMEFTHiggs_decays.hpp
 *
 * @brief contains class for calculating particle decays
 *
 * This file was generated with FlexibleSUSY 2.6.1 and SARAH 4.14.3 .
 */

#ifndef MSSMEFTHiggs_DECAYS_H
#define MSSMEFTHiggs_DECAYS_H

#include "MSSMEFTHiggs_decay_table.hpp"
#include "MSSMEFTHiggs_mass_eigenstates.hpp"
#include "MSSMEFTHiggs_mass_eigenstates_decoupling_scheme.hpp"
#include "cxx_qft/MSSMEFTHiggs_qft.hpp"
#include "MSSMEFTHiggs_decay_amplitudes.hpp"
#include "decays/flexibledecay_problems.hpp"
#include "lowe.h"
#include "wrappers.hpp"
#include "error.hpp"
#include "physical_input.hpp"
#include "decays/flexibledecay_settings.hpp"

namespace flexiblesusy {

template <typename Field1, typename Field2>
constexpr std::enable_if_t<!std::is_same<Field1, Field2>::value, double>
final_state_symmetry_factor(typename MSSMEFTHiggs_cxx_diagrams::field_indices<Field1>::type const& idx1,
                            typename MSSMEFTHiggs_cxx_diagrams::field_indices<Field2>::type const& idx2)
{
   return 1.;
}

template <typename Field1, typename Field2>
std::enable_if_t<std::is_same<Field1, Field2>::value, double>
final_state_symmetry_factor(typename MSSMEFTHiggs_cxx_diagrams::field_indices<Field1>::type const& idx1,
                            typename MSSMEFTHiggs_cxx_diagrams::field_indices<Field2>::type const& idx2)
{
   if (boost::range::equal(idx1, idx2)) {
      return 0.5;
   }
   else {
      return 1.;
   }
}

class MSSMEFTHiggs_decays {
public:
   MSSMEFTHiggs_decays() = default;
   MSSMEFTHiggs_decays(MSSMEFTHiggs_mass_eigenstates model_, softsusy::QedQcd const& qedqcd_,
         Physical_input const& physical_input_,
         FlexibleDecay_settings const& flexibledecay_settings_)
      : model(model_)
      , qedqcd(qedqcd_)
      , physical_input(physical_input_)
      , flexibledecay_settings(flexibledecay_settings_)
      {}
   MSSMEFTHiggs_decays(const MSSMEFTHiggs_decays&) = default;
   MSSMEFTHiggs_decays(MSSMEFTHiggs_decays&&) = default;
   ~MSSMEFTHiggs_decays() = default;
   MSSMEFTHiggs_decays& operator=(const MSSMEFTHiggs_decays&) = default;
   MSSMEFTHiggs_decays& operator=(MSSMEFTHiggs_decays&&) = default;

   const MSSMEFTHiggs_decay_table& get_decay_table() const;
   const FlexibleDecay_problems& get_problems() const;

   void clear();
   void clear_problems();
   void calculate_decays();

   const Decays_list& get_hh_decays(int i) const { return decay_table.
      get_hh_decays(i); }
   const Decays_list& get_Ah_decays(int i) const { return decay_table.
      get_Ah_decays(i); }
   const Decays_list& get_Hpm_decays(int i) const { return decay_table.
      get_Hpm_decays(i); }
   const Decays_list& get_Su_decays(int i) const { return decay_table.
      get_Su_decays(i); }
   const Decays_list& get_Sd_decays(int i) const { return decay_table.
      get_Sd_decays(i); }
   const Decays_list& get_Se_decays(int i) const { return decay_table.
      get_Se_decays(i); }
   const Decays_list& get_Sv_decays(int i) const { return decay_table.
      get_Sv_decays(i); }
   void calculate_hh_decays();
   void calculate_Ah_decays();
   void calculate_Hpm_decays();
   void calculate_Su_decays();
   void calculate_Sd_decays();
   void calculate_Se_decays();
   void calculate_Sv_decays();

double partial_width_hh_to_SdconjSd(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_SvconjSv(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_SuconjSu(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_SeconjSe(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_hhhh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_hhAh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_AhAh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_HpmconjHpm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_hhVP(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_hh_to_AhVP(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_hh_to_hhVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_hh_to_AhVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_hh_to_conjHpmVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_hh_to_HpmconjVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_hh_to_VGVG(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_hh_to_VPVP(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_hh_to_VPVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_hh_to_VZVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_hh_to_conjVWmVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_hh_to_GluGlu(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_hh_to_barFvFv(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_ChiChi(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_barChaCha(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_barFeFe(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_barFdFd(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_hh_to_barFuFu(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_SdconjSd(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_SvconjSv(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_SuconjSu(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_SeconjSe(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_hhhh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_HpmconjHpm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_hhVP(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Ah_to_hhVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Ah_to_conjHpmVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Ah_to_HpmconjVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Ah_to_VGVG(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_Ah_to_VPVP(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_Ah_to_VPVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_Ah_to_VZVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_Ah_to_conjVWmVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_Ah_to_GluGlu(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_Ah_to_barFvFv(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_ChiChi(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_barChaCha(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_barFeFe(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_barFdFd(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Ah_to_barFuFu(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Hpm_to_SdconjSu(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Hpm_to_SeconjSv(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Hpm_to_hhVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Hpm_to_AhVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Hpm_to_VPVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_Hpm_to_VZVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int) const;
double partial_width_Hpm_to_ChiCha(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Hpm_to_barFvFe(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Hpm_to_barFuFd(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Su_to_SdconjHpm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Su_to_Suhh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Su_to_SuAh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Su_to_SuVG(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Su_to_SuVP(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Su_to_SuVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Su_to_SdconjVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Su_to_GluFu(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Su_to_ChiFu(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Su_to_barChaFd(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sd_to_Sdhh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sd_to_SdAh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sd_to_SuHpm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sd_to_SdVG(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Sd_to_SdVP(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Sd_to_SdVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Sd_to_SuVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Sd_to_GluFd(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Sd_to_ChiFd(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sd_to_ChaFu(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Se_to_SvHpm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Se_to_Sehh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Se_to_SeAh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Se_to_SeVP(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Se_to_SeVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Se_to_SvVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Se_to_FvCha(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Se_to_ChiFe(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sv_to_Svhh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sv_to_SvAh(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sv_to_SeconjHpm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sv_to_SvVP(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Sv_to_SvVZ(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Sv_to_SeconjVWm(MSSMEFTHiggs_mass_eigenstates_interface*, int, int) const;
double partial_width_Sv_to_FvChi(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;
double partial_width_Sv_to_barChaFe(MSSMEFTHiggs_mass_eigenstates_interface*, int, int, int) const;

private:
   MSSMEFTHiggs_mass_eigenstates model{};
   softsusy::QedQcd qedqcd{};
   Physical_input physical_input;
   FlexibleDecay_settings flexibledecay_settings {};
   bool run_to_decay_particle_scale {true};
   MSSMEFTHiggs_decay_table decay_table{};
   FlexibleDecay_problems problems{};

   template<typename FieldIn, typename FieldOut1, typename FieldOut2>
   typename Decay_amplitude_type<FieldIn, FieldOut1, FieldOut2>::type
   calculate_amplitude(
      const MSSMEFTHiggs_cxx_diagrams::context_base&,
      const typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldIn>::type&,
      const typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut1>::type&,
      const typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut2>::type&) const;

   template<typename FieldIn, typename FieldOut1, typename FieldOut2>
   double amplitude_squared(MSSMEFTHiggs_cxx_diagrams::context_base const& context,
                  typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldIn>::type const& indexIn,
                  typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut1>::type const& indexOut1,
                  typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut2>::type const& indexOut2) const;

   template<typename FieldIn, typename FieldOut1, typename FieldOut2>
   double get_partial_width(
      const MSSMEFTHiggs_cxx_diagrams::context_base&,
      typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldIn>::type const&,
      typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut1>::type const&,
      typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut2>::type const&) const;
};

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Sd, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sd>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sd>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Sv, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sv>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sv>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Su, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Su>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Su>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Se, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Se>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Se>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::hh>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Ah>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Ah>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Hpm, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Hpm, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VG, MSSMEFTHiggs_cxx_diagrams::fields::VG>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VP, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VP, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VZ, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Glu, MSSMEFTHiggs_cxx_diagrams::fields::Glu>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Glu >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Glu >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fv>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fv>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fv>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fv >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::Chi, MSSMEFTHiggs_cxx_diagrams::fields::Chi>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Cha>::type, MSSMEFTHiggs_cxx_diagrams::fields::Cha>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Cha>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Cha >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fe>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fe>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fe>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fe >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fd>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fd>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fd>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fd >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::hh, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fu>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fu >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Sd, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sd>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sd>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Sv, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sv>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sv>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Su, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Su>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Su>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Se, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Se>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Se>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::hh>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Hpm, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Hpm, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::VG, MSSMEFTHiggs_cxx_diagrams::fields::VG>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::VP, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::VP, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::VZ, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Glu, MSSMEFTHiggs_cxx_diagrams::fields::Glu>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Glu >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Glu >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fv>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fv>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fv>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fv >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::Chi, MSSMEFTHiggs_cxx_diagrams::fields::Chi>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Cha>::type, MSSMEFTHiggs_cxx_diagrams::fields::Cha>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Cha>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Cha >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fe>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fe>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fe>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fe >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fd>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fd>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fd>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fd >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Ah, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fu>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fu >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, MSSMEFTHiggs_cxx_diagrams::fields::Sd, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Su>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Su>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, MSSMEFTHiggs_cxx_diagrams::fields::Se, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sv>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Sv>::type >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, MSSMEFTHiggs_cxx_diagrams::fields::hh, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, MSSMEFTHiggs_cxx_diagrams::fields::Ah, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, MSSMEFTHiggs_cxx_diagrams::fields::VP, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SVV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, MSSMEFTHiggs_cxx_diagrams::fields::VZ, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, MSSMEFTHiggs_cxx_diagrams::fields::Chi, MSSMEFTHiggs_cxx_diagrams::fields::Cha>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Cha >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fv>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fe>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fv>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fe >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Hpm, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fd>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fd >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Sd, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::hh>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Ah>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::VG>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Sd, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Glu, MSSMEFTHiggs_cxx_diagrams::fields::Fu>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Glu >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fu >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Chi, MSSMEFTHiggs_cxx_diagrams::fields::Fu>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fu >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Su, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Cha>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fd>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Cha>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fd >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::hh>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Ah>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::Hpm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::VG>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Su, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Su >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Glu, MSSMEFTHiggs_cxx_diagrams::fields::Fd>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Glu >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fd >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Chi, MSSMEFTHiggs_cxx_diagrams::fields::Fd>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fd >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sd, MSSMEFTHiggs_cxx_diagrams::fields::Cha, MSSMEFTHiggs_cxx_diagrams::fields::Fu>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sd >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Cha >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fu >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Hpm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Hpm >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::hh>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Ah>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::VWm>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Fv, MSSMEFTHiggs_cxx_diagrams::fields::Cha>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Cha >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Se, MSSMEFTHiggs_cxx_diagrams::fields::Chi, MSSMEFTHiggs_cxx_diagrams::fields::Fe>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fe >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::hh>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Ah>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&) const;

template<>
Decay_amplitude_SSS MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Se, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::Hpm>::type >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::VP>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::VZ>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;

template<>
Decay_amplitude_SSV MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Se, typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Se >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sv, MSSMEFTHiggs_cxx_diagrams::fields::Fv, MSSMEFTHiggs_cxx_diagrams::fields::Chi>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Chi >::type&) const;

template<>
Decay_amplitude_SFF MSSMEFTHiggs_decays::calculate_amplitude<MSSMEFTHiggs_cxx_diagrams::fields::Sv, typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Cha>::type, MSSMEFTHiggs_cxx_diagrams::fields::Fe>(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Sv >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Cha>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fe >::type&) const;


template<typename FieldIn, typename FieldOut1, typename FieldOut2>
double
MSSMEFTHiggs_decays::amplitude_squared(MSSMEFTHiggs_cxx_diagrams::context_base const& context,
                  typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldIn>::type const& indexIn,
                  typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut1>::type const& indexOut1,
                  typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut2>::type const& indexOut2) const
{

   const auto mat_elem = calculate_amplitude<FieldIn, FieldOut1, FieldOut2>(
      context, indexIn, indexOut1, indexOut2);
   return mat_elem.square();
}

// returns a squared color generator for a 3 point amplitude with FieldIn, FieldOut1 and FieldOut2
// averaged over inital state colors
// the generator is guessed from color representations of FieldIn, FieldOut1 and FieldOut2
// This is not a bulletproof solution and might fail in general but is enough for
// decays of color singlets

// 1 -> 1, 1
template<typename FieldIn, typename FieldOut1, typename FieldOut2>
constexpr
std::enable_if_t<
MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldIn> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldOut2>, double>
squared_color_generator() {return 1.;}

// 1 -> 3, 3bar
template<typename FieldIn, typename FieldOut1, typename FieldOut2>
constexpr
std::enable_if_t<
   MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldIn>
   &&
   (
   (MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldOut1> &&
   MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldOut2>)
   ||
   (MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldOut1> &&
   MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldOut2>)
   ), double
>
squared_color_generator() {return 3.;}

// 1 -> 8, 8
template<typename FieldIn, typename FieldOut1, typename FieldOut2>
constexpr
std::enable_if_t<
MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldIn> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldOut2>, double>
squared_color_generator() {return 8.;}

// 3 -> 3, 1; 3bar -> 3bar, 1
template<typename FieldIn, typename FieldOut1, typename FieldOut2>
constexpr
std::enable_if_t<
(
MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldIn> &&
((MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldOut2>) ||
(MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldOut2>))
)
||
(
MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldIn> &&
((MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldOut2>) ||
(MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldOut2>))
)
, double>
squared_color_generator() {return 1.;}

// 3 -> 3, 8; 3bar -> 3bar, 8
template<typename FieldIn, typename FieldOut1, typename FieldOut2>
constexpr
std::enable_if_t<
(
MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldIn> &&
((MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldOut2>) ||
(MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldOut2>))
)
||
(
MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldIn> &&
((MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldOut2>) ||
(MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldOut2>))
)
, double>
squared_color_generator() {return 4.;}

// 8 -> 8, 1
template<typename FieldIn, typename FieldOut1, typename FieldOut2>
constexpr
std::enable_if_t<
MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldIn> &&
((MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldOut2>) ||
(MSSMEFTHiggs_cxx_diagrams::fields::is_singlet_v<FieldOut1> &&
MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldOut2>))
, double>
squared_color_generator() {return 1.;}

// 8 -> 3, 3bar
template<typename FieldIn, typename FieldOut1, typename FieldOut2>
constexpr
std::enable_if_t<
   MSSMEFTHiggs_cxx_diagrams::fields::is_octet_v<FieldIn>
   &&
   (
   (MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldOut1> &&
   MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldOut2>)
   ||
   (MSSMEFTHiggs_cxx_diagrams::fields::is_anti_triplet_v<FieldOut1> &&
   MSSMEFTHiggs_cxx_diagrams::fields::is_triplet_v<FieldOut2>)
   ), double
>
squared_color_generator() {return 1./2.;}

// generic decay of FieldIn -> FieldOut1 FieldOut2
template<typename FieldIn, typename FieldOut1, typename FieldOut2>
double MSSMEFTHiggs_decays::get_partial_width(
   const MSSMEFTHiggs_cxx_diagrams::context_base& context,
   typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldIn>::type const& indexIn,
   typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut1>::type const& indexOut1,
   typename MSSMEFTHiggs_cxx_diagrams::field_indices<FieldOut2>::type const& indexOut2
   ) const
{

   // on-shell masses
   const double mIn = context.physical_mass<FieldIn>(indexIn);
   const double mOut1 = context.physical_mass<FieldOut1>(indexOut1);
   const double mOut2 = context.physical_mass<FieldOut2>(indexOut2);

   // is decay kinematically allowed?
   if(mIn < mOut1 + mOut2) {
      WARNING("Called kinematically forbidden decay");
      return 0.;
   }

   // phase space without symmetry factor
   const double ps = 1./(8.*Pi) * std::sqrt(KallenLambda(1., Sqr(mOut1/mIn), Sqr(mOut2/mIn)));

   // phase space symmetry factor
   const double ps_symmetry =
      final_state_symmetry_factor<FieldOut1, FieldOut2>(indexOut1, indexOut2);

   // color factor
   constexpr double color_factor = squared_color_generator<FieldIn, FieldOut1, FieldOut2>();

   // matrix element squared
   const auto mat_elem_sq = amplitude_squared<FieldIn, FieldOut1, FieldOut2>(
      context, indexIn, indexOut1, indexOut2);

   // flux * phase space factor * symmetry factor * color factor * |matrix element|^2
   const auto result = 0.5/mIn * ps * ps_symmetry * color_factor * mat_elem_sq;

   return result;
}

template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::hh,MSSMEFTHiggs_cxx_diagrams::fields::VZ,MSSMEFTHiggs_cxx_diagrams::fields::VZ >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::hh,typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type,MSSMEFTHiggs_cxx_diagrams::fields::VWm >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::conj<MSSMEFTHiggs_cxx_diagrams::fields::VWm>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VWm >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::hh,MSSMEFTHiggs_cxx_diagrams::fields::VG,MSSMEFTHiggs_cxx_diagrams::fields::VG >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::hh,MSSMEFTHiggs_cxx_diagrams::fields::VP,MSSMEFTHiggs_cxx_diagrams::fields::VP >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::hh,MSSMEFTHiggs_cxx_diagrams::fields::VP,MSSMEFTHiggs_cxx_diagrams::fields::VZ >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::hh,typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type,MSSMEFTHiggs_cxx_diagrams::fields::Fu >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fu >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::hh,typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fd>::type,MSSMEFTHiggs_cxx_diagrams::fields::Fd >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fd>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fd >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::hh,typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fe>::type,MSSMEFTHiggs_cxx_diagrams::fields::Fe >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::hh >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fe>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fe >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::Ah,typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fd>::type,MSSMEFTHiggs_cxx_diagrams::fields::Fd >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fd>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fd >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::Ah,typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type,MSSMEFTHiggs_cxx_diagrams::fields::Fu >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fu>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fu >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::Ah,MSSMEFTHiggs_cxx_diagrams::fields::VG,MSSMEFTHiggs_cxx_diagrams::fields::VG >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VG >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::Ah,MSSMEFTHiggs_cxx_diagrams::fields::VP,MSSMEFTHiggs_cxx_diagrams::fields::VP >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::Ah,MSSMEFTHiggs_cxx_diagrams::fields::VP,MSSMEFTHiggs_cxx_diagrams::fields::VZ >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VP >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::VZ >::type&) const;
template <>
double MSSMEFTHiggs_decays::get_partial_width<MSSMEFTHiggs_cxx_diagrams::fields::Ah,typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fe>::type,MSSMEFTHiggs_cxx_diagrams::fields::Fe >(const MSSMEFTHiggs_cxx_diagrams::context_base&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Ah >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<typename MSSMEFTHiggs_cxx_diagrams::fields::bar<MSSMEFTHiggs_cxx_diagrams::fields::Fe>::type >::type&, const typename MSSMEFTHiggs_cxx_diagrams::field_indices<MSSMEFTHiggs_cxx_diagrams::fields::Fe >::type&) const;

} // namespace flexiblesusy

#endif
